#include "EngineList.h"
#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>

EngineList::EngineList(bool sorted)
{
	isSorted=sorted;
}

EngineList::EngineList(const EngineList& el)
{
	operator=(el);
}

EngineList& EngineList::operator =(const EngineList& el)
{
	isSorted=el.isSorted;
	list=el.list;
	return *this;
}

void EngineList::add(EngineList &el)
{
	for(vector<Engine>::iterator it = (el.list).begin(); it!=(el.list).end(); ++it)
		list.push_back(*it);
	if(isSorted) sort();
}


void EngineList::add(Engine &e)
{
	list.push_back(e);
	if(isSorted) sort();
}

void EngineList::remove(unsigned int index)
{
	list.erase(list.begin()+index);
}

void EngineList::clear()
{
	list.clear();
}

unsigned int EngineList::size()
{
	return list.size();
}

Engine& EngineList::operator [](unsigned int index)
{
	return list[index];
}

Engine& EngineList::get(unsigned int index)
{
	return list[index];
}

void EngineList::sort()
{
	std::sort(list.begin(), list.end());
}

int EngineList::find(Engine& e)
{
	for(vector<Engine>::iterator it = list.begin(); it!=list.end(); ++it)
		if(it->name==e.name && it->exe==e.exe && it->parameters== e.parameters)
			return it-list.begin();
	return -1;
}

#define LD_MATRIX(X,Y) *(matrix+(m+1)*(X)+(Y))
unsigned int Levenshtein_Distance(wxString s1, wxString s2)
{
    unsigned int n = s1.size();
    unsigned int m = s2.size();
    if(n==0 || m==0)
        return (unsigned int)-1;
    s1.LowerCase();
    s2.LowerCase();
    unsigned int i,j;
    unsigned int* matrix = (unsigned int *)malloc(sizeof(unsigned int)*(n+1)*(m+1));


    for(j=0;j<=m;j++) LD_MATRIX(0,j) = j;
    for(i=0;i<=n;i++) LD_MATRIX(i,0) = i;
    for(i=1;i<=n;i++)
        for(j=1;j<=m;j++)
        {
            unsigned int a = LD_MATRIX(i-1,j)+1;
            unsigned int b = LD_MATRIX(i,j-1)+1;
            unsigned int c = LD_MATRIX(i-1,j-1)+(unsigned int)(s1.GetChar(i-1)!=s2.GetChar(j-1));
            if(a<b)
            {
                if(a<c) LD_MATRIX(i,j) = a;
                else LD_MATRIX(i,j) = c;
            }
            else
            {
                if(b<c) LD_MATRIX(i,j) = b;
                else LD_MATRIX(i,j) = c;
            }
        }
    unsigned int distance = LD_MATRIX(n,m);
    free(matrix);
    return distance;
}

int EngineList::bestMatch(wxString& name)
{
    int best = -1;
    unsigned int bestDifference = (unsigned int)-1;
    int i=0;
    	for(vector<Engine>::iterator it = list.begin(); it!=list.end(); ++it, ++i)
    	{
            unsigned int difference = Levenshtein_Distance(name,it->name);
            if(difference<bestDifference)
            {
                best = i;
                bestDifference = difference;
            }
        }
    return best;
}

void EngineList::updateControl(wxControlWithItems& lb)
{
	wxArrayString nItems;
	for(vector<Engine>::iterator it = list.begin(); it!=list.end(); ++it)
		nItems.Add((*it).name);
	lb.Clear();
	lb.Append(nItems);
}

bool EngineList::readDefaultFile()
{
	FILE* f=fopen(DEFAULT_ENGINE_LIST_FILE, "r");
	char line[MAX_LINE_LENGTH];
	
	if(f==NULL) return true;
	while(fgets(line, MAX_LINE_LENGTH, f))
	{
		wxString name(line);
		if(!fgets(line, MAX_LINE_LENGTH, f)) goto end;
		wxString exe=line;
		if(!fgets(line, MAX_LINE_LENGTH, f)) goto end;
		wxString parameters=line;
		name.RemoveLast();
		exe.RemoveLast();
		parameters.RemoveLast();
		Engine e(name,exe,parameters);
		list.push_back(e);
	}
end:
	if(isSorted) sort();
	fclose(f);
	return false;
}

bool EngineList::writeDefaultFile()
{
	FILE* f=fopen(DEFAULT_ENGINE_LIST_FILE, "w");
	if(f==NULL) return true;
	for(vector<Engine>::iterator it = list.begin(); it!=list.end(); ++it)
		fprintf(f,"%s\n%s\n%s\n",(it->name).c_str(),((it->exe).GetFullPath()).c_str(),(it->parameters).c_str());
	fclose(f);
	return false;
}

bool EngineList::readTournamentFile()
{
	FILE* f=fopen(TOURNAMENT_ENGINE_LIST_FILE, "r");
	char line[MAX_LINE_LENGTH];
	
	if(f==NULL) return true;
	while(fgets(line, MAX_LINE_LENGTH, f))
	{
		wxString name(line);
		if(!fgets(line, MAX_LINE_LENGTH, f)) goto end;
		wxString exe=line;
		if(!fgets(line, MAX_LINE_LENGTH, f)) goto end;
		wxString parameters=line;
		name.RemoveLast();
		exe.RemoveLast();
		parameters.RemoveLast();
		Engine e(name,exe,parameters);
		list.push_back(e);
	}
end:
	if(isSorted) sort();
	fclose(f);
	return false;
}

bool EngineList::writeTournamentFile()
{
	FILE* f=fopen(TOURNAMENT_ENGINE_LIST_FILE, "w");
	if(f==NULL) return true;
	for(vector<Engine>::iterator it = list.begin(); it!=list.end(); ++it)
		fprintf(f,"%s\n%s\n%s\n",(it->name).c_str(),((it->exe).GetFullPath()).c_str(),(it->parameters).c_str());
	fclose(f);
	return false;
}

bool EngineList::read(const wxString& fullFilePath)
{
	FILE* f=fopen(fullFilePath.c_str(), "r");
	char line[MAX_LINE_LENGTH];
	wxFileName fn(fullFilePath);
	
	if(f==NULL) return true;
	while(fgets(line, MAX_LINE_LENGTH, f))
	{
		wxString name(line);
		if(!fgets(line, MAX_LINE_LENGTH, f)) goto end;
		wxString exe=line;
		if(!fgets(line, MAX_LINE_LENGTH, f)) goto end;
		wxString parameters=line;
		name.RemoveLast();
		exe.RemoveLast();
		parameters.RemoveLast();
		exe=fn.GetPath()+exe;
		Engine e(name,exe,parameters);
		list.push_back(e);
	}
end:
	if(isSorted) sort();
	fclose(f);
	return false;
	
}

bool EngineList::readWBINI(const wxString& fullFilePath)
{
	FILE* f=fopen(fullFilePath.c_str(), "r");
	if(f==NULL) return true;
	char line[MAX_LINE_LENGTH];
	while(fgets(line, MAX_LINE_LENGTH, f))
	{
		wxString theLine(line);
		if(theLine.StartsWith("/firstChessProgramNames={", &theLine))
		{
			do
			{
				char exe[MAX_LINE_LENGTH];
				char dir[MAX_LINE_LENGTH];
				char name[MAX_LINE_LENGTH];
				char params[MAX_LINE_LENGTH];
				*exe='\0';
				*dir='\0';
				*name='\0';
				*params='\0';
				sscanf(theLine.c_str(),"\"%[^\"]\" /fd=\"%[^\"]\"%*[^;]; /name=\"%[^\"]\"",exe,dir,name);
				sscanf(exe,"%*s %[^\"]",params);
				sscanf(exe,"%s",exe);
				wxString executable(exe);
				wxString directory(dir);
				wxString enginename(name);
				wxString parameters(params);
				Engine e(enginename, directory+"/"+executable+".exe", parameters);
				list.push_back(e);
				if(!fgets(line, MAX_LINE_LENGTH, f))
					break;
				theLine=line;
			}while(!theLine.Contains("}"));
		}
	}
	fclose(f);
	if(isSorted) sort();
	return false;
}

bool EngineList::write(const wxString& fullFilePath)
{
	FILE* f=fopen(fullFilePath.c_str(), "w");
	wxFileName fn(fullFilePath);
	
	if(f==NULL) return true;
	for(vector<Engine>::iterator it = list.begin(); it!=list.end(); ++it)
	{
		wxString relativePath;
		if((*it).isInDir(fn.GetPath(), &relativePath))
		fprintf(f,"%s\n%s\n%s\n",(it->name).c_str(),relativePath.c_str(),(it->parameters).c_str());
	}
	fclose(f);
	return false;
}
