#include "Tournament.h"
#include "Configuration.h"
#include <wx/filename.h>
#include <wx/colour.h>

#ifndef ID_PSWBTM_MAIN_WINDOW
#define ID_PSWBTM_MAIN_WINDOW 1000
#endif

BEGIN_EVENT_TABLE(Tournament, wxEvtHandler)
	EVT_END_PROCESS(wxID_ANY, Tournament::OnProcessTerm)
END_EVENT_TABLE()

Tournament::Tournament()
:proc(this)
{
	isValid=false;
	running=false;
	status=STATUS_NONE;
	pauseEveryRound=false;
	parent=NULL;
	IPtype=INITIAL_POSITION_DEFAULT;
}

Tournament::Tournament(wxEvtHandler* daddy)
:proc(this)
{
	isValid=false;
	running=false;
	status=STATUS_NONE;
	pauseEveryRound=false;
	parent=daddy;
	IPtype=INITIAL_POSITION_DEFAULT;
}


Tournament::Tournament(wxString PGN1, wxString RTPGN1, wxString DEBUGFOLDER1, int type1, int gamesPerPairing1, int timeControlType, int TV1, int TV2, bool pondering1, EngineList& participants1, Engine& GauntletEngine1, int setDepth1, wxString TN, int IP_type, wxString IP_file,  bool IP_twogames, int IP_rewindAfter)
:proc(this)
{
	isValid=true;
	PGN=PGN1;
	RTPGN=RTPGN1;
	DEBUGFOLDER=DEBUGFOLDER1;
	type=type1;
	gamesPerPairing=gamesPerPairing1;
	TCtype=timeControlType;
	timeVariable1=TV1;
	timeVariable2=TV2;
	timeVariable3=0; //Edit this
	pondering=pondering1;
	Participants=EngineList(EngineList::UNSORTED);
	Participants.add(participants1);
	GauntletEngine=GauntletEngine1;
	currentPairing=0;
	running=false;
	status=STATUS_NONE;
	gamesLeft=gamesPerPairing;
	setDepth=setDepth1;
	pauseEveryRound=false;
	TournamentName=TN;
	parent=NULL;
	timeTaken=0;
	scores.Alloc(Participants.size());
	games.Alloc(Participants.size());
	scores.Add(0,Participants.size());
	games.Add(0,Participants.size());
	IPtype=IP_type;
	IPfile=IP_file;
	IPtwoGames=IP_twogames;
	IPrewindAfter=IP_rewindAfter;
	//createPairings();
}

unsigned int Tournament::rounds()
{
	switch(type)
	{
		case GAUNTLET:
			return 1;
		case ROUND_ROBIN:
			if(Participants.size()%2)
				return Participants.size();
			else
				return Participants.size()-1;
		default:
			return 0;
	}
}

int Tournament::roundNum(int gameNum)
{
	switch(type)
	{
		case GAUNTLET:
			return 0;
		case ROUND_ROBIN:
			return (gameNum/pairingsPerRound());
		default:
			return 0;
	}
}

int Tournament::subroundNum(int gameNum)
{
	return gameNum-roundNum(gameNum)*pairingsPerRound();
}

int Tournament::pairingsPerRound()
{
	switch(type)
	{
		case GAUNTLET:
			return ((int)Participants.size());
		case ROUND_ROBIN:
			if(Participants.size()%2)
				return (Participants.size()-1)/2;
			else return Participants.size()/2;
		default:
			return 0;
	}
}

void Tournament::read()
{
	isValid=false;
	Participants=EngineList(EngineList::UNSORTED);
	if(Participants.readTournamentFile())
		return;
	FILE* f=fopen(TOURNAMENT_FILE, "r");
	if(f==NULL) return;
	
	char str[4096];
	int temp;
	wxString temp2;
	fgets(str,4096,f);
	TournamentName=str;
	TournamentName.RemoveLast();
	fgets(str,4096,f);
	PGN=str;
	PGN.RemoveLast();
	fgets(str,4096,f);
	RTPGN=str;
	RTPGN.RemoveLast();
	fgets(str,4096,f);
	DEBUGFOLDER=str;
	DEBUGFOLDER.RemoveLast();
	fgets(str,4096,f);
	GauntletEngine.name=str;
	(GauntletEngine.name).RemoveLast();
	fgets(str,4096,f);
	temp2=str;
	temp2.RemoveLast();
	GauntletEngine.exe=temp2;
	fgets(str,4096,f);
	GauntletEngine.parameters=str;	
	(GauntletEngine.parameters).RemoveLast();
	
	fgets(str,4096,f);
    IPfile=str;
    IPfile.RemoveLast();
	
	fscanf(f,"%d",&IPtype);
	fscanf(f,"%d",&type);
	fscanf(f,"%d",&gamesPerPairing);
	
	fscanf(f,"%d",&TCtype);
	fscanf(f,"%d",&timeVariable1);
	fscanf(f,"%d",&timeVariable2);
	fscanf(f,"%d",&timeVariable3);
	fscanf(f,"%d",&temp);
	pondering=(bool)temp;
	fscanf(f,"%d",&setDepth);
	fscanf(f,"%d",&gamesLeft);
	fscanf(f,"%d",&currentPairing);
	fscanf(f,"%d",&temp);
	IPtwoGames = temp;
	fscanf(f,"%d",&IPrewindAfter);
	timeTaken=0;
	unsigned long tempTimeTaken;
	fscanf(f,"%x",&tempTimeTaken);
	timeTaken|=((unsigned long long)(tempTimeTaken))<<32;
	fscanf(f,"%x",&tempTimeTaken);
	timeTaken|=(unsigned long long)(tempTimeTaken);
	
	int i;
	FILE* gamesf=fopen(GAMES_FILE, "rb");
	if(gamesf==NULL) return;
	FILE* scoresf=fopen(SCORES_FILE, "rb");
	if(scoresf==NULL)
	{
		fclose(gamesf);
		return;
	}
	games.Clear();
	scores.Clear();
	for(i=0;i<(int)Participants.size();i++)
	{
		int a;
		fread(&a,sizeof(int),1,gamesf);
		games.Add(a);
		fread(&a,sizeof(int),1,scoresf);
		scores.Add(a);
	}
	fclose(scoresf);
	fclose(gamesf);
	
	fclose(f);
	if(PGN!=wxEmptyString)
		isValid=true;
	pauseEveryRound=false;
	createPairings();
}

void Tournament::write()
{
	if(!isValid) return;
	Participants.writeTournamentFile();
	FILE* f=fopen(TOURNAMENT_FILE, "w");
	if(f==NULL) return;
	fprintf(f,"%s\n",TournamentName.c_str());
	fprintf(f,"%s\n",PGN.c_str());
	fprintf(f,"%s\n",RTPGN.c_str());
	fprintf(f,"%s\n",DEBUGFOLDER.c_str());
	fprintf(f,"%s\n%s\n%s\n",(GauntletEngine.name).c_str(),((GauntletEngine.exe).GetFullPath()).c_str(),(GauntletEngine.parameters).c_str());
	fprintf(f,"%s\n%d\n",IPfile.c_str(),IPtype);
    fprintf(f,"%d\n",type);
	fprintf(f,"%d\n",gamesPerPairing);
	fprintf(f,"%d\n",TCtype);
	fprintf(f,"%d\n",timeVariable1);
	fprintf(f,"%d\n",timeVariable2);
	fprintf(f,"%d\n",timeVariable3);
    fprintf(f,"%d\n",pondering);
	fprintf(f,"%d\n",setDepth);
	fprintf(f,"%d\n",gamesLeft);
	fprintf(f,"%d\n",currentPairing);
	fprintf(f,"%d\n",(int)IPtwoGames);
	fprintf(f,"%d\n",IPrewindAfter);
	fprintf(f,"%x\n%x\n",(unsigned long)(timeTaken>>32), (unsigned long)(timeTaken&0xFFFFFFFF));
	fclose(f);
	
	int i;
	FILE* gamesf=fopen(GAMES_FILE, "wb");
	if(gamesf==NULL) return;
	FILE* scoresf=fopen(SCORES_FILE, "wb");
	if(scoresf==NULL)
	{
		fclose(gamesf);
		return;
	}
	for(i=0;i<(int)Participants.size();i++)
	{
		int a=games.Item(i);
		fwrite(&a,sizeof(int),1,gamesf);
		a=scores.Item(i);
		fwrite(&a,sizeof(int),1,scoresf);
	}
	fclose(scoresf);
	fclose(gamesf);
}

void Tournament::createPairings()
{
	pairings.clear();
	EngineList tempList=Participants;
	unsigned int i;
	FILE* f=fopen("pairings.txt","w");
	if(type==ROUND_ROBIN)
	{
		if(tempList.size()%2)
		{
			Engine dummy(wxEmptyString,wxEmptyString,wxEmptyString);
			tempList.add(dummy);
		}
		unsigned int j;
		for(i=0;i<rounds();i++)
		{
			fprintf(f,"\nRound %d\n",i+1);
			int k=0;
			for(j=0;j<tempList.size()/2;j++)
			{
				Engine fcp;
				Engine scp;
				if(i%2 && j==0)
				{
					fcp=tempList[tempList.size()-j-1];
					scp=tempList[j];	
				}
				else
				{
					fcp=tempList[j];
					scp=tempList[tempList.size()-j-1];
				}
				if(fcp.name==wxEmptyString || scp.name==wxEmptyString)
					continue;
				Pairing p(fcp,scp);
				pairings.push_back(p);
				k++;
				fprintf(f,"%d) %s - %s\n",k,(fcp.name).c_str(),scp.name.c_str());
			}
			unsigned int rotations;
			for(rotations=0;rotations<tempList.size()/2;rotations++)
			{
				Engine e1=tempList[tempList.size()-1];
				tempList.remove(tempList.size()-1);
				Engine e2=tempList[0];
				tempList.remove(0);
				tempList.add(e2);
				tempList.add(e1);
			}
		}
	}
	else //Gauntlet
	{
		fprintf(f,"%s Gauntlet\n",(GauntletEngine.name).c_str());
		for(i=0;i<tempList.size();i++)
		{
			if(i%2)
			{
				pairings.push_back(Pairing(GauntletEngine,tempList[i]));
				fprintf(f,"%d) %s - %s\n",i+1,(GauntletEngine.name).c_str(),(tempList[i].name).c_str());
			}
			else
			{
				pairings.push_back(Pairing(tempList[i],GauntletEngine));
				fprintf(f,"%d) %s - %s\n",i+1,(tempList[i].name).c_str(),(GauntletEngine.name).c_str());
			}
		}
	}
	fclose(f);
}

void Tournament::updatePairingDisplay(wxControlWithItems& r, wxListCtrl& lc, int round)
{
	unsigned int i;
	lc.DeleteAllItems();
	
	if(type==GAUNTLET)
	{
		r.Clear();
		r.Append("Gauntlet");
		for(i=0;i<Participants.size();i++)
		{
			wxString str;
			str.Printf("%d",i+1);
			lc.InsertItem(i, str);
			lc.SetItem(i,1,(pairings[i].fcp).name);
			lc.SetItem(i,2,(pairings[i].scp).name);
		}
		goto end;
	}
	else //Round Robin
	{
		if(r.GetCount()!=rounds())
		{
			r.Clear();
			for(i=0;i<rounds();i++)
			{
				
				wxString str;
				str.Printf("Round %d",i+1);
				r.Append(str);
			}
		}
		
		for(int(i)=round*pairingsPerRound();int(i)<(round+1)*pairingsPerRound();i++)
		{
			wxString str;
			str.Printf("%d",i-round*pairingsPerRound()+1);
			lc.InsertItem(i-round*pairingsPerRound(), str);
			lc.SetItem(i-round*pairingsPerRound(),1,(pairings[i].fcp).name);
			lc.SetItem(i-round*pairingsPerRound(),2,(pairings[i].scp).name);
		}
	}
end:
	wxColour veryLIGHT_GREY(240,240,240);
	if(round<roundNum(currentPairing))
	{
		for(i=0;int(i)<pairingsPerRound();i++)
			lc.SetItemBackgroundColour(i, veryLIGHT_GREY);
	}
	else if(round==roundNum(currentPairing))
	{
		for(i=0;int(i)<subroundNum(currentPairing);i++)
			lc.SetItemBackgroundColour(i, veryLIGHT_GREY);
		lc.SetItemBackgroundColour(subroundNum(currentPairing), *wxLIGHT_GREY);
		lc.EnsureVisible(subroundNum(currentPairing));
	}
}

void Tournament::updateDetails(wxTextCtrl& tc)
{
	tc.Clear();
	
	if(TournamentName!=wxEmptyString)
		tc << "Event: " << TournamentName << "\n";
	
	if(type==GAUNTLET)
		tc << GauntletEngine.name << " Gauntlet\n";
	else //Round Robin
		tc << "Round Robin\n";
	if(gamesPerPairing>1)
		tc << gamesPerPairing << " games per pairing\n";
	tc << "Time Control: ";
	
	if(TCtype==TIME_CONTROL_FIXED)
		tc << "Fixed - " << timeVariable1 << " seconds\n";
	else if(TCtype==TIME_CONTROL_INCREMENT)
		tc << "Increment - " << timeVariable1 << " minutes + " << timeVariable2 << " second increment\n";
	else
		tc << timeVariable2 << " moves in " << timeVariable1 << " minutes\n";
	if(setDepth>0)
		tc << "Depth is limited to " << setDepth <<"\n";
	if(pondering)
		tc << "Pondering is on\n";
	else
		tc << "Pondering is off\n";
	switch(IPtype)
	{
        case Tournament::INITIAL_POSITION_DEFAULT:
            break;
        case Tournament::INITIAL_POSITION_FEN:
            tc << "Starting Positions: " << IPfile << "\n";
            break;
        case Tournament::INITIAL_POSITION_PGN:
            tc << "Starting Moves: " << IPfile << "\n";
            break;
    }
	tc << int(Participants.size()) << " participants\n\n";
	
	if(type==GAUNTLET)
	{
		tc << "Pairing " << currentPairing+1 << "\n";
		tc << "Game " << (gamesPerPairing-gamesLeft+1) << "\n\n";
	}
	else //Round Robin
	{
		tc << "Round " << roundNum(currentPairing)+1 << "\n";
		tc << "Sub-Round " << subroundNum(currentPairing)+1 << "\n";
		tc << "Game " << (gamesPerPairing-gamesLeft+1) << "\n\n";
	}
	
	int totalGamesPlayed=gamesPerPairing*(currentPairing+1)-gamesLeft;
	int totalGamesLeft=(int)(gamesPerPairing*(pairings.size()-1 - currentPairing)+gamesLeft);
	
	tc << "Games Played: " << totalGamesPlayed <<"\n";
	tc << "Games Left: " << totalGamesLeft << "\n\n";

	if(timeTaken)
	{
		unsigned long long tempTime=timeTaken;
		tc << "Playing time:";
		if(tempTime >(86400000))
		{
			tc << " " << long(tempTime/86400000) << " days";
			tempTime%=86400000;
		}
		if(tempTime >(3600000))
		{
			tc << " " << long(tempTime/3600000) << " hours";
			tempTime%=3600000;
		}
		if(tempTime >(60000))
		{
			tc << " " << long(tempTime/60000) << " minutes";
			tempTime%=60000;
		}
		if(tempTime >(1000))
		{
			tc << " " << long(tempTime/1000) << " seconds";
			tempTime%=1000;
		}
		tc << "\n";
		
		if(totalGamesPlayed)
		{
			tempTime=(timeTaken/totalGamesPlayed);
			tc << "Average time per game:";
			if(tempTime >(86400000))
			{
				tc << " " << long(tempTime/86400000) << " days";
				tempTime%=86400000;
			}
			if(tempTime >(3600000))
			{
				tc << " " << long(tempTime/3600000) << " hours";
				tempTime%=3600000;
			}
			if(tempTime >(60000))
			{
				tc << " " << long(tempTime/60000) << " minutes";
				tempTime%=60000;
			}
			if(tempTime >(1000))
			{
				tc << " " << long(tempTime/1000) << " seconds";
				tempTime%=1000;
			}
			tc << "\n";
			tempTime=(timeTaken*totalGamesLeft/totalGamesPlayed);
			tc << "Estimated time to finish:";
			if(tempTime >(86400000))
			{
				tc << " " << long(tempTime/86400000) << " days";
				tempTime%=86400000;
			}
			if(tempTime >(3600000))
			{
				tc << " " << long(tempTime/3600000) << " hours";
				tempTime%=3600000;
			}
			if(tempTime >(60000))
			{
				tc << " " << long(tempTime/60000) << " minutes";
				tempTime%=60000;
			}
			if(tempTime >(1000))
			{
				tc << " " << long(tempTime/1000) << " seconds";
				tempTime%=1000;
			}
			tc << "\n";
		}
		tc << "\n";
	}

	//tc << "Winboard Parameters: " << WBParameters() << "\n\n";
	tc << "PGN File: " << PGN << "\n";
	if(RTPGN!=wxEmptyString)
		tc << "Result Table PGN File: " << RTPGN << "\n";
	if(DEBUGFOLDER!=wxEmptyString)
		tc << "Debug Folder: " << DEBUGFOLDER << "\n";
}

void Tournament::run()
{	
	if(running) return;
	wxRemoveFile("temp.pgn.pswbtm");
	wxRemoveFile("temp2.pgn.pswbtm");
	wxRemoveFile("winboard.debug");
	FILE* f=fopen(CONFIG_FILE,"r");
	char wbExe[4096];
	if(f==NULL)
	{
		wxMessageBox(wxString("Please go to the configurations dialog and update the location of the Winboard/Xboard executable."),wxString("Winboard/Xboard not found!")/*,wxOk|wxICON_ERROR*/);
		running=false;
		updateDaddy();
		return;
	}
	if(!fgets(wbExe,4096,f))
	{
		fclose(f);
		wxMessageBox(wxString("Please go to the configurations dialog and update the location of the Winboard/Xboard executable."),wxString("Winboard/Xboard not found!")/*,wxOk|wxICON_ERROR*/);
		running=false;
		updateDaddy();
		return;
	}
	fclose(f);
	wxFileName fn;
	fn=(wxString(wbExe).RemoveLast());

    #ifdef _WIN32
	if(!(fn.IsOk() && fn.FileExists()))
	{
		wxMessageBox(wxString("Please go to the configurations dialog and update the location of the Winboard/Xboard executable."),wxString("Winboard/Xboard not found!")/*,wxOk|wxICON_ERROR*/);
		running=false;
		updateDaddy();
		return;
	}
	#endif
	
	running=true;
	status=Tournament::STATUS_NONE;
	updateDaddy();
	sw.Start();
	//wxMessageBox(fn.GetFullPath()+" "+WBParameters()+" "+pairings[currentPairing].getWBparameters(gamesPerPairing-gamesLeft));
	pid=wxExecute(fn.GetFullPath()+" "+WBParameters()+" "+pairings[currentPairing].getWBparameters(gamesPerPairing-gamesLeft),wxEXEC_ASYNC|wxEXEC_MAKE_GROUP_LEADER,&proc);
}

void Tournament::killPause()
{
	if(!running) return;
	status=Tournament::STATUS_KILL_PAUSE;
	wxProcess::Kill(pid,wxSIGKILL,wxKILL_CHILDREN);
	wxRemoveFile("temp.pgn.pswbtm");
	wxRemoveFile("temp2.pgn.pswbtm");
	wxRemoveFile("winboard.debug");
	return;
}
void Tournament::finish()
{
	isValid=false;
	//Operations After Game
	FILE* aftergame=fopen("aftertourney.txt","rt");
	if(aftergame!=NULL)
	{
		char line[4096];
		while(fgets(line,4096,aftergame))
		{
			wxString Line(line);
			if(Line.Last()=='\n')
				Line.RemoveLast();
			wxExecute(Line,wxEXEC_SYNC);
		}
	}
	wxRemoveFile("pairings.txt");
	wxRemoveFile(TOURNAMENT_FILE);
	wxRemoveFile(GAMES_FILE);
	wxRemoveFile(SCORES_FILE);
	wxRemoveFile("temp.pgn.pswbtm");
	wxRemoveFile("temp2.pgn.pswbtm");
	wxRemoveFile("winboard.debug");
	wxRemoveFile(TOURNAMENT_ENGINE_LIST_FILE);
	updateDaddy();
}

void Tournament::OnProcessTerm(wxProcessEvent& p)
{
	running=false;
	switch(status)
	{
		case STATUS_KILL_PAUSE:
			status=STATUS_NONE;
			updateDaddy();
			return;
		case STATUS_ABORT:
			status=STATUS_NONE;
			finish();
			return;
	}
	
	
	//Update PGNs and Debug info
	
	//Open Temp PGN
	FILE* tempPGN=fopen("temp.pgn.pswbtm","r");
	if(tempPGN!=NULL)
	{
        //Operations After Game
	    FILE* aftergame=fopen("aftergame.txt","rt");
	    if(aftergame!=NULL)
 	    {
	    	char line[4096];
	     	while(fgets(line,4096,aftergame))
	      	{
	       		wxString Line(line);
       			if(Line.Last()=='\n')
   	         		Line.RemoveLast();
        		wxExecute(Line,wxEXEC_SYNC);
         	}
       	}
	
        timeTaken+=(unsigned long long)(sw.Time());
		Engine whiteE=pairings[currentPairing].White(gamesPerPairing-gamesLeft);
		Engine blackE=pairings[currentPairing].Black(gamesPerPairing-gamesLeft);
		int whiteEngineIndex=Participants.find(whiteE);
		int blackEngineIndex=Participants.find(blackE);
		//replace name tags and round and number
		FILE* tempPGN2=fopen("temp2.pgn.pswbtm","w");
		{
			char line[4096];
			bool numTag=false;
			while(fgets(line,4096,tempPGN))
			{
				wxString wxLine=line;
				if(wxLine.StartsWith("[White"))
					sprintf(line,"[White \"%s\"]\n",(whiteE.name).c_str());
				else if(wxLine.StartsWith("[Black"))
					sprintf(line,"[Black \"%s\"]\n",(blackE.name).c_str());
				else if(wxLine.StartsWith("[Round "))
					sprintf(line,"[Round \"%d.%d\"]\n",roundNum(currentPairing)+1,subroundNum(currentPairing)+1);
				else if(wxLine.StartsWith("[Result"))
				{
					games[whiteEngineIndex]++;
					games[blackEngineIndex]++;
					if(wxLine.Contains("1-0"))
						scores[whiteEngineIndex]+=2;
					else if(wxLine.Contains("0-1"))
						scores[blackEngineIndex]+=2;
					else if(wxLine.Contains("1/2-1/2"))
					{
						scores[whiteEngineIndex]++;
						scores[blackEngineIndex]++;
					}
				}
				else if(wxLine=="\n" && !numTag)
				{
					numTag=true;
					sprintf(line,"[Number \"%d\"]\n\n",gamesPerPairing*(currentPairing+1)-gamesLeft+1);
				}
				fprintf(tempPGN2,line);
			}
		}
		
		//Close Temp PGN
		fclose(tempPGN);
		wxRemoveFile("temp.pgn.pswbtm");
		fclose(tempPGN2);
		
		//Append to PGN and Result Table PGN
		tempPGN2=fopen("temp2.pgn.pswbtm","r");
		FILE* PGNfile = fopen(PGN.c_str(),"a");
		FILE* RTPGNfile = NULL;
		if(RTPGN!=wxEmptyString);
		RTPGNfile = fopen(RTPGN.c_str(),"a");
		{
			char line[4096];
			while(fgets(line,4096,tempPGN2))
			{
				fprintf(PGNfile,line);
				if(RTPGNfile!=NULL)
					fprintf(RTPGNfile, line);
			}
		}
		if(RTPGNfile!=NULL) fclose(RTPGNfile);
		fclose(PGNfile);
		fclose(tempPGN2);
		wxRemoveFile("temp2.pgn.pswbtm");
		//Finished the Append
		
		#ifdef OLIVIER_VERSION
            tries_left=NUM_TRIES;
        #endif
	}
	else
	{
        #ifdef OLIVIER_VERSION
            if(tries_left)
            {
                tries_left--;
                run();
                return;
            }
        #endif
        
		if(wxYES==wxMessageBox(wxString("The game ended without starting. Would you like to pause?"),wxString("Error..."),wxYES_NO))
		{
			status=STATUS_NONE;
			updateDaddy();
			return;
		}
	}
	//Place the debug file in the debug folder and name it accordingly
	if(DEBUGFOLDER!=wxEmptyString)
	{
		wxString debugfile;
		debugfile.Printf("/%d.debug",gamesPerPairing*(currentPairing+1)-gamesLeft+1);
		debugfile=DEBUGFOLDER+debugfile;
		wxCopyFile(wxString("winboard.debug"),debugfile);
		wxRemoveFile("winboard.debug");
	}
	//Finished replacing debug files
	
	gamesLeft--;
	if(gamesLeft==0)
	{
		gamesLeft=gamesPerPairing;
		currentPairing++;
		if(currentPairing>=(int)pairings.size())
		{
			gamesLeft=0;
			finish();
			return;
		}
	}
	
	//Write Tournament Files
	write();
	
	if(status==STATUS_PENDING_PAUSE)
	{
		status=STATUS_NONE;
		updateDaddy();
		return;
	}
	if(pauseEveryRound && type!=GAUNTLET && currentPairing!=0 && gamesLeft==gamesPerPairing)
	{
		if(currentPairing%pairingsPerRound()==0)
		{
            status=STATUS_NONE;
			updateDaddy();
			return;
		}
	}
	run();
}

void Tournament::updateDaddy()
{
	if(parent==NULL) return;
	wxCommandEvent event(wxEVT_UPDATE_INFO,ID_PSWBTM_MAIN_WINDOW);
    event.SetEventObject(this);
    event.SetString(wxString("Update All"));
    parent->ProcessEvent(event);
	return;
}

void Tournament::makeDaddy(wxEvtHandler* daddy)
{
	parent=daddy;
}

wxString Tournament::WBParameters()
{
	wxString params="-saveSettingsOnExit false -sgf \"temp.pgn.pswbtm\" -autosave -popupExitMessage false -cp -thinking -xperiodic -mg 1 -xreuse -xreuse2";
	if(TournamentName!=wxEmptyString)
		params+=" -pgnEventHeader \""+TournamentName+"\"";
	if(pondering)
		params+=" -ponder";
	else
		params+=" -xponder";
	if(DEBUGFOLDER!=wxEmptyString)
		params+=" -debug";
	if(setDepth)
	{
		params+=" -depth ";
		params+=wxString::Format("%d", setDepth);
	}
	switch(TCtype)
	{
		case TIME_CONTROL_FIXED:
			params+=" -st ";
			params+=wxString::Format("%d", timeVariable1/60);
			params+=":";
			params+=wxString::Format("%d", timeVariable1%60);
		break;
		case TIME_CONTROL_INCREMENT:
			params+=" -tc ";
			params+=wxString::Format("%d", timeVariable1);
			params+=" -inc ";
			params+=wxString::Format("%d", timeVariable2);
		break;
		case TIME_CONTROL_XY:
            params+=" -movesPerSession ";
			params+=wxString::Format("%d", timeVariable2);
			params+=" -tc ";
			params+=wxString::Format("%d", timeVariable1);
		break;
	}
	int IPnum;
	switch(IPtype)
	{
        case Tournament::INITIAL_POSITION_DEFAULT:
            IPnum=0;
            break;
        default:
            IPnum = (gamesPerPairing-gamesLeft);
            if(IPtwoGames) IPnum /= 2;
            if(IPrewindAfter) IPnum%=IPrewindAfter;      
    }
    IPnum+=1;
	switch(IPtype)
	{
        case Tournament::INITIAL_POSITION_DEFAULT:
            break;
        case Tournament::INITIAL_POSITION_FEN:
            params+= " -lpf \"";
            params+= IPfile;
            params+= "\" -lpi ";
            params+=wxString::Format("%d",IPnum);
            break;
        case Tournament::INITIAL_POSITION_PGN:
            params+= " -lgf \"";
            params+= IPfile;
            params+= "\" -lgi ";
            params+=wxString::Format("%d",IPnum);
            break;
    }
	return params;
}


int wxCALLBACK updateStandingsHelper(long item1, long item2, long sortData)
{
	if(item1==item2) return 0;
	if(item1>item2) return -1;
	return 1;
}

void Tournament::updateStandings(wxListCtrl& lc)
{
	lc.DeleteAllItems();
	int i;
	for(i=0;i<int(Participants.size());i++)
	{
		wxString str;
		str.Printf("%d",i+1);
		lc.InsertItem(i, str);
		
		lc.SetItem(i,1,Participants[i].name);
		
		str.Printf("%d",scores[i]/2);
		if(scores[i]%2)
			str+=".5";
		lc.SetItem(i,2,str);
		
		str.Printf("%d",games[i]);
		lc.SetItem(i,3,str);
		
		if(games[i]!=0)
			str.Printf("%.1f",100*(scores[i]/(2*float(games[i]))));
		else
		    str=wxEmptyString;
        lc.SetItem(i,4,str);
        lc.SetItemData(i, scores[i]);
	}
	lc.SortItems(updateStandingsHelper,0);
	for(i=0;i<int(Participants.size());i++)
	{
		wxString str;
		str.Printf("%d",i+1);
		lc.SetItemText(i, str);
	}
}
