#ifndef __Tournament
#define __Tournament

#include "Engine.h"
#include "EngineList.h"
#include "Pairing.h"
#include "UpdateInfoEvt.h"
#include <stdio.h>
#include <wx/utils.h>
#include <wx/process.h>
#include <wx/event.h>
#include <wx/ctrlsub.h>
#include <wx/listctrl.h>
#include <wx/textctrl.h>
#include <wx/stopwatch.h>
#include <wx/dynarray.h>
#include <vector>

using std::vector;

#define TOURNAMENT_FILE "tournament.pswbtm"
#define SCORES_FILE "scores.pswbtm"
#define GAMES_FILE "games.pswbtm"

#define OLIVIER_VERSION
#ifdef OLIVIER_VERSION
    #define NUM_TRIES 3
#endif

class Tournament : public wxEvtHandler
{
	public:
        #ifdef OLIVIER_VERSION
            int tries_left;
        #endif
		bool isValid;
		//GUI Pointer
		wxEvtHandler* parent;
		
		//Name!
		wxString TournamentName;
		
		//File Settings
		wxString PGN;
		wxString RTPGN;
		wxString DEBUGFOLDER;
		
		//Tournament Type
		static const int GAUNTLET = 0;
		static const int ROUND_ROBIN = 1;
		int type;
		int gamesPerPairing;
		Engine GauntletEngine;
		
		//Time/Thinking Control
		static const int TIME_CONTROL_FIXED = 0;
		static const int TIME_CONTROL_INCREMENT = 1;
		static const int TIME_CONTROL_XY = 2;
		int TCtype;
		int timeVariable1; //Fixed-seconds, Increment/XY - Initial minutes
		int timeVariable2; //Increment-increment XY-moves until new time control
		int timeVariable3; //Increment/XY-seconds time
		int setDepth;
		bool pondering;
		
		//Initial Position Settings
		static const int INITIAL_POSITION_DEFAULT = 0;
		static const int INITIAL_POSITION_FEN = 1;
		static const int INITIAL_POSITION_PGN = 2;
		int IPtype;
		wxString IPfile;
		int IPrewindAfter;
		bool IPtwoGames;
		
		EngineList Participants;
		wxArrayInt scores;
		wxArrayInt games;
		
		//Own variables
		
		//Tournament Status
		vector<Pairing> pairings;
		int gamesLeft;
		int currentPairing;
		unsigned long long timeTaken;
		wxStopWatch sw;
		
		//Process Control
		wxProcess proc;
		int pid;
		bool running;
		static const int STATUS_NONE = 0;
		static const int STATUS_PENDING_PAUSE = 1;
		static const int STATUS_KILL_PAUSE = 2;
		static const int STATUS_ABORT = 4;
		int status;
		bool pauseEveryRound;
		
		Tournament();
		Tournament(wxEvtHandler* daddy);
		Tournament(wxString PGN1, wxString RTPGN1, wxString DEBUGFOLDER1, int type1, int gamesPerPairing1, int timeControlType, int TV1, int TV2, bool pondering1, EngineList& participants1, Engine& GauntletEngine1, int setDepth1, wxString TN=wxEmptyString, int IP_type=INITIAL_POSITION_DEFAULT, wxString IP_File=wxEmptyString, bool IP_twogames=false, int IP_rewindAfter=0);
		virtual ~Tournament(){}
		unsigned int rounds();
		int roundNum(int gameNum);
		int subroundNum(int gameNum);
		int pairingsPerRound();
		void createPairings();
		void read();
		void write();
		void updatePairingDisplay(wxControlWithItems& rounds, wxListCtrl& lc, int round);
		void updateStandings(wxListCtrl& lc);
		void run();
		void killPause();
		void finish();
		void OnProcessTerm(wxProcessEvent& p);
		void updateDaddy();
		void makeDaddy(wxEvtHandler* daddy);
		wxString WBParameters();
		void updateDetails(wxTextCtrl& tc);
	DECLARE_EVENT_TABLE();
};

#endif
